using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Resources;
using System.Windows.Forms;

using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Tools.XPMenus;

using UBS.A4.GUI;
using UBS.A4.GUI.Configuration;

namespace UBS.A4.Windows.Forms
{

    //TODO: ultimatly,  this class should implement dockable form interface 
	public class A4DockableForm : A4Form
	{
        private const string CLASS_NAME = "A4DockableForm";

        protected MainFrameBarManager mainFrameBarManager1;
        protected PopupMenusManager popupMenusManager1;
        
        protected A4DockableFormConfiguration mA4DockableFormConfig = null;
        protected DockingManager    mDockingManager;

		/*public MainFrameBarManager BarManager
		{
			get{return mainFrameBarManager1;}
		}
		*/
        /// <summary>
        /// ctor for designer purposes only
        /// </summary>
        public A4DockableForm() : base()
        {
            InitializeComponent();
        }

        public A4DockableForm( string id ) : this( id, null, null )
        {
        }

        public A4DockableForm( string id, string idToCopyFrom ) : this( id, idToCopyFrom, null )
        {
        }

        
        public A4DockableForm( string id, string idToCopyFrom, DockingManager dockingManager ) : base( id, idToCopyFrom )
        {
            InitializeComponent();

            // we control the bar!
            mainFrameBarManager1.EnableCustomizing = false;
            
            // tell our base about our config elements
            base.AddConfigType( typeof(A4DockableFormConfiguration) );
            mDockingManager = dockingManager;
        }

        private new void InitializeComponent()
        {
            this.SuspendLayout();
            ResourceManager resources = new ResourceManager(typeof(A4DockableForm));
            this.mainFrameBarManager1 = new MainFrameBarManager(this.components, this);
            this.popupMenusManager1 = new PopupMenusManager(this.components);
            ((ISupportInitialize)(this.mainFrameBarManager1)).BeginInit();
            
            // 
            // mainFrameBarManager1
            // 
            this.mainFrameBarManager1.AutoLoadToolBarPositions = false;
            this.mainFrameBarManager1.AutoPersistCustomization = false;
            this.mainFrameBarManager1.BarPositionInfo = ((MemoryStream)(resources.GetObject("mainFrameBarManager1.BarPositionInfo")));
            this.mainFrameBarManager1.CurrentBaseFormType = "UBS.A4.Windows.Forms.A4DockableForm";
            this.mainFrameBarManager1.Form = this;
            this.mainFrameBarManager1.ImageList = null;
            this.mainFrameBarManager1.LargeImageList = null;

            ((ISupportInitialize)(this.mainFrameBarManager1)).EndInit();
            this.ResumeLayout(false);

        }

        public override void ReadConfiguration( IConfigurationService configuration )
        {
            base.ReadConfiguration( configuration );

            // if there ain't one of us in then add it
            // note: there will only ever be one a4dockable form
            // config element in our base's config
            // don't worry about uniqueness on id's here
            if( mA4FormConfig.ConfigurationElement( CLASS_NAME ) == null )
            {
                // create a new un and add it
                mA4DockableFormConfig = 
                    new A4DockableFormConfiguration( mA4FormConfig, CLASS_NAME, base.Left, base.Top, 
                    base.Width, base.Height, DockPosition.top, DockState.floating );
            }
            else
                mA4DockableFormConfig = (A4DockableFormConfiguration)mA4FormConfig.ConfigurationElement( CLASS_NAME );

            // prevents the ever shrinking window syndrome
            if( mA4DockableFormConfig.DockState == DockState.docked )
                this.Height += 34;
            else
                this.Height += 8;
        }
   
        /// <summary>
        /// Adds the given toolitems to the given toolbar,
        /// ensures that all items are correctly added to the menu manager
        /// </summary>
        /// <param name="toolItems"></param>
        /// <param name="toolBar"></param>
            protected void addToolItems( IToolbarItem[] toolItems, Bar toolBar )
            {
                foreach( IToolbarItem item in toolItems )
                {
                   
                    if( item != null && 
                        item.ToolbarItems != null )
                    {
                        // a parent
                        ParentBarItem newPBI = new ParentBarItem();                    
                        newPBI.ID = Guid.NewGuid().ToString();
                        newPBI.Popup += new EventHandler( toolItemClick );
                        newPBI.CategoryIndex = 0;
                        newPBI.Tag = item;
                        newPBI.Text = item.Caption;
                        newPBI.Visible = item.Visible;
                        newPBI.Tooltip = item.Description;
                        if( item.Command != null )
                            newPBI.Enabled = item.Command.CanExecute;
                        else
                            newPBI.Enabled = item.Enabled;

                        mainFrameBarManager1.Items.Add( newPBI );

                        // store a ref to this baritem
                        item.ToolbarItemRef = newPBI;
            
                        toolBar.Items.Add( newPBI );

                        // add the parent's items to it
                        addToolItems( item.ToolbarItems, newPBI );
                    }
                    else
                    if( item != null )
                    {
                        // a regular item
                        BarItem bi = new BarItem();                    
                        bi.ID = Guid.NewGuid().ToString();
                        bi.CategoryIndex = 0;
                        bi.Click += new EventHandler( toolItemClick );
                        bi.Tag = item;
                        bi.Text = item.Caption;
                        if( item.Command != null )
                        {
                            bi.Checked = item.Command.HasExecuted;
                            bi.Enabled = item.Command.CanExecute;
                        }
                        else
                            bi.Enabled = item.Enabled;
                        mainFrameBarManager1.Items.Add( bi );                    
                        
                        item.ToolbarItemRef = bi;

                        toolBar.Items.Add( bi );
                    }
                }
                
            }
            /// <summary>
            /// adds the given toolitems to the given parent baritem
            /// ensures that all items are correctly added to the menu manager
            /// </summary>
            /// <param name="toolItems"></param>
            /// <param name="pbi"></param>
            public void addToolItems( IToolbarItem[] toolItems, ParentBarItem pbi )
            {
                foreach( IToolbarItem item in toolItems )
                {
                   
                    if( item != null && 
                        item.ToolbarItems != null )
                    {
                        // a parent
                        ParentBarItem newPBI = new ParentBarItem();
                        newPBI.ID = Guid.NewGuid().ToString();
                        newPBI.CategoryIndex = 0;
                        newPBI.Tag = item;
                        newPBI.Text = item.Caption;
                        newPBI.Visible = item.Visible;
                        newPBI.Tooltip = item.Description;
                        newPBI.Enabled = item.Enabled;
                        mainFrameBarManager1.Items.Add( newPBI );

                        // store a ref to this baritem
                        item.ToolbarItemRef = newPBI;
            
                        pbi.Items.Add( newPBI );
                        addToolItems( item.ToolbarItems, newPBI );
                    }
                    else
                    if( item != null )
                    {
                        // a regular item
                        BarItem bi = new BarItem();
                        bi.ID = Guid.NewGuid().ToString();
                        bi.CategoryIndex = 0;
                        bi.Click += new EventHandler( toolItemClick );
                        bi.Tag = item;
                        bi.Text = item.Caption;
                        if( item.Command != null )
                        {
                            bi.Checked = item.Command.HasExecuted;
                            bi.Enabled = item.Command.CanExecute;
                        }
                        else
                            bi.Enabled = item.Enabled;
                        mainFrameBarManager1.Items.Add( bi );                    
                        
                        item.ToolbarItemRef = bi;

                        pbi.Items.Add( bi );
                    }
                }
            }

            // copied from mainform - please don't change this!
            protected void toolItemClick( object sender, EventArgs e )
            {
                try
                {

                    if( sender.GetType() == typeof( BarItem ))
                    {

                        // call the command function
                        BarItem bi = (BarItem)sender;

                        IToolbarItem ti = bi.Tag as IToolbarItem;
                        if( ti != null )
                            if( ti.Command != null )
                            {
                                if( ti.Command.CanExecute )
                                {
                                    ICommandReturn r = ti.Command.Execute();
                                    //trace.assert( r != null, "no command return from " + ti.Command.GetType().ToString() );
                                    if( r != null &&
                                        r.Result == CommandReturnCode.Failed )
                                        MessageBox.Show( (IWin32Window)this, r.ResultText, ti.Command.GetType().ToString(), MessageBoxButtons.OK, MessageBoxIcon.Exclamation );
                                }
                            }
                        else
                                Trace.WriteLine( "toolItemClick: toolbar has no IToolbarItem object, " + bi.Text, CLASS_NAME );
                    }
                    else
                    if( sender.GetType() == typeof( ParentBarItem ))
                    {

                        // call the command function
                        ParentBarItem pbi = (ParentBarItem)sender;

                        IToolbarItem ti = pbi.Tag as IToolbarItem;
                        if( ti != null )
                        {
                            if( ti.Command != null )
                            {
                                //TODO: handle the response 
                                ti.Command.Execute();

                                // re-init the items contained within the parent
                                ArrayList items = new ArrayList( pbi.Items );


                                foreach( BarItem item in items )
                                {
                                    pbi.Items.Remove( item );
                                    mainFrameBarManager1.Items.Remove( item );
                                }   

                                // put 'em back
                                addToolItems( ti.ToolbarItems, pbi );
                            }
                            else
                            {
                                foreach( BarItem item in pbi.Items )
                                {
                                    ti = item.Tag as IToolbarItem;
                                    if( ti == null )
                                        continue;
                                    ICommand command = ti.Command;
                                    if( command == null )
                                        continue;

                                    item.Checked = command.HasExecuted;
                                    item.Enabled = command.CanExecute;
                                }
                            }
                        }
                        

                    }
                }
                catch( Exception ex )
                {
                    Trace.WriteLine( "toolItemClick: Exception " + ex, CLASS_NAME );
                    Trace.Flush();

                    MessageBox.Show( (IWin32Window)this, "An error occured processing this command, please see log", "Toolbar", MessageBoxButtons.OK, MessageBoxIcon.Exclamation );
                }
            }

        public A4DockableFormConfiguration A4DockableFormConfiguration
        {
            get { return mA4DockableFormConfig; }
        }

        protected override void OnResize(EventArgs e)
        {
            if( mA4DockableFormConfig == null )
            {
                base.OnResize( e );
                return;
            }
            // supress resize information if we are pinned and hidden
            if( mA4DockableFormConfig.DockState == DockState.pinned )
                base.OnResize( e, true );
            else
                base.OnResize( e );


        }

        public DockingManager DockingManager
        {
            get { return mDockingManager; }
            set { mDockingManager = value; }
        }

        public bool IsFloating
        {
            get
            {
                if( mDockingManager != null )
                    return mDockingManager.IsFloating( this );
                else
                    return false;
            }
        }

        public void DockControl()
        {
        }

        /// <summary>
        /// when overridded, shows/hides the auto hide button
        /// </summary>
        /// <param name="visible"></param>
        public virtual void SetAutoHideButtonVisibility( bool visible )
        {
            mDockingManager.SetAutoHideButtonVisibility( this, visible );
        }

        /// <summary>
        /// when overridded, shows/hides the close button
        /// </summary>
        /// <param name="visible"></param>
        public virtual void SetCloseButtonVisibility( bool visible )
        {
            mDockingManager.SetCloseButtonVisibility( this, visible );
        }

        /// <summary>
        /// asks the docking manager to activate me
        /// </summary>
        protected virtual void DMActivate()
        {
            if( mDockingManager != null )
                mDockingManager.ActivateControl( this );
        }
       
        /// <summary>
        /// Makes this form visible. If 'pinned' this form will show itself
        /// </summary>
        protected void MakeVisible()
        {

            if( this.InvokeRequired )
                this.Invoke( new MethodInvoker( MakeVisible ));
            else
            {                
                try
                {
                    base.VisibleNoActivate();
                    // don't do this if I'm not pinned
                    // this is 'cos the makevisible sets the focus, which is bloody annoying when
                    // loads o'things are going into the notifications window
                    if( this.A4DockableFormConfiguration.DockState == DockState.pinned )
                        DMActivate();
                    else
                    if( this.A4DockableFormConfiguration.DockState == DockState.floating )
                        this.TopMostNoActivate = true;

                }
                catch( Exception e )
                {
                    Trace.WriteLine( "MakeVisible: Exception " + e, CLASS_NAME );
                    Trace.Flush();
                }
                catch
                {
                    Trace.WriteLine( "MakeVisible: Internal Exception ", CLASS_NAME );
                    Trace.Flush();
                }
            }
        }
    }
}

