using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;

using Syncfusion.Runtime.Serialization;
using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Tools.Design;

using UBS.A4.GUI;

namespace UBS.A4.Windows.Forms
{
	/// <summary>
	/// Summary description for DockHostControl.
	/// </summary>
	public class DockHostControl : UserControl
	{
		private readonly static string CLASS_NAME;

		static DockHostControl()
		{
			CLASS_NAME = typeof(DockHostControl).ToString();
		}

        private class FixedDockingManager : DockingManager
        {

            public FixedDockingManager( Container container ) : base( container )
            {
            }

            public bool AddDockedControl( Control ctrl, DockingStyle dockStyle )
            {
                if( this.alEnableDocking.Contains( ctrl ))
                    return false;

                this.alEnableDocking.Add( ctrl );

                InternalEnableDocking( ctrl, dockStyle, -1 );

                if((this.DesignMode == false) && (this.alEnableDocking.Count > 0))
                {
                    if(DesignerHooks.CallWndProcListContains(this) == false)
                        DesignerHooks.AddCallWndProcListener(this);
                    if(DesignerHooks.GetMsgProcListContains(this) == false)
                        DesignerHooks.AddGetMsgProcListener(this);
                }

                return true;
            }


            public override void DockControl(Control ctrl, Control parent, DockingStyle dockstyle, int nsize)
            {
                if( parent == this.HostControl && dockstyle != DockingStyle.Tabbed )
                {
                    if(this.alEnableDocking.Contains(ctrl) == false)
                        this.alEnableDocking.Add(ctrl);
                    
                    this.InternalDockControl(ctrl, this.dcHostForm, dockstyle, nsize);				
                }
                else
                    base.DockControl( ctrl, parent, dockstyle, nsize );
            }

        }

		/// <summary> 
		/// Required designer variable.
		/// </summary>
		protected Container components = null;
        private FixedDockingManager mDockingManager;
        private bool                mFrozen = false;

		public DockHostControl(  )
		{

			InitializeComponent();
            InitializeDockingManager(  );
        }

        private void InitializeDockingManager( )
        {
            mDockingManager = new FixedDockingManager( this.components );
            //mDockingManager.BeginInit();
            mDockingManager.HostControl = this;
            //mDockingManager.EndInit();

            mDockingManager.DockStateUnavailable += new DockStateUnavailableEventHandler( mDockingManager_DockStateUnavailable );
            mDockingManager.DockAllow += new DockAllowEventHandler( mDockingManager_DockAllow );
            mDockingManager.DragAllow += new DragAllowEventHandler(mDockingManager_DragAllow);
        }

        public bool CloseEnabled
        {
            get { return mDockingManager.CloseEnabled; }
            set { mDockingManager.CloseEnabled = value; }
        }

        public void LoadState( Stream stream )
        {
            AppStateSerializer serializer = new AppStateSerializer( SerializeMode.XMLFmtStream, stream );
			stream.Seek(0, SeekOrigin.Begin);
            mDockingManager.LoadDockState( serializer );
        }

        public void SaveState( Stream stream )
        {
            AppStateSerializer serializer = new AppStateSerializer( SerializeMode.XMLFmtStream, stream );
            mDockingManager.SaveDockState( serializer );
            serializer.PersistNow();
        }

        public DockingManager DockingManager
        {
            get { return mDockingManager; }
        }

        public void AddDockedControl( string dockLabel, Control control )
        {
            mDockingManager.SetEnableDocking( control, true );
            mDockingManager.SetDockLabel( control, dockLabel );
        }

        public void AddDockableForm( A4DockableForm form, DockPosition position )
        {
            Trace.WriteLine( "Dockhost: " + this.Name + "Adding dockable form named: " + form.Name );
            DockingStyle dockStyle;

            switch( position )
            {
                case DockPosition.left:     dockStyle = DockingStyle.Left; break;
                case DockPosition.right:    dockStyle = DockingStyle.Right; break;
                case DockPosition.bottom:   dockStyle = DockingStyle.Bottom; break;
                default:                    dockStyle = DockingStyle.Top; break;
            }

            Size size = form.Size;
            size.Height = size.Height <= 10 ? 75 : size.Height;

            //mDockingManager.SetEnableDocking( form, true );
            //mDockingManager.AddDockedControl( form, dockStyle );

            switch( form.A4DockableFormConfiguration.DockState )
            {
                case GUI.DockState.pinned:
                    form.Size = new Size( form.A4DockableFormConfiguration.Width, form.A4DockableFormConfiguration.Height );
                    mDockingManager.DockControl( form, this, dockStyle, form.A4DockableFormConfiguration.Width );
					mDockingManager.SetAutoHideMode( form, true );	
                    //mDockingManager.SetControlSize( form, new Size( form.A4DockableFormConfiguration.Height, form.A4DockableFormConfiguration.Width));
                    break;

                case GUI.DockState.docked:
                    mDockingManager.DockControl( form, this, dockStyle, form.Configuration.Height );
                    mDockingManager.SetAutoHideMode( form, false );
                    //mDockingManager.SetControlSize( form, size );
                    break;

                case GUI.DockState.floating:
                    mDockingManager.FloatControl( form, new Rectangle( form.Top, form.Left, size.Width, size.Height ));
                    break;

                default:
                    mDockingManager.AddDockedControl( form, dockStyle );
                    break;
            }

            form.DockingManager = mDockingManager;
            mDockingManager.SetDockLabel( form, form.Text );
            form.TextChanged += new EventHandler(control_TextChanged);
        }            

        /// <summary>
        /// AddDockedControl adds a control to the docking manager.
        /// The control's docking label is automagically populated based
        /// on the control.Text propery.  Also the dock label will follow the
        /// Control.Text propery based on the TextChanged event
        /// </summary>
        /// <param name="control"></param>
        public void AddDockedControl( Control control )
        {
            mDockingManager.SetEnableDocking( control, true );
            mDockingManager.SetDockLabel( control, control.Text );

            control.TextChanged += new EventHandler(control_TextChanged);
        }

        public void DockAllWindows()
        {
            IEnumerator controls = mDockingManager.Controls;

            while( controls.MoveNext() )
            {
                try
                {
                    Control control = (Control) controls.Current;

                    if( mDockingManager.IsFloating( control ))
                    {
                        DockHost dhost = control.Parent as DockHost;
                        DockHostController dhc = (DockHostController) dhost.InternalController;
                        dhc.TransitToPrevDock();
                    }
                }
                catch {}
            }
        }

        public bool Frozen
        {
            get { return mFrozen; }
            set
            {
                if( mFrozen == value )
                    return;

                mFrozen = value;
            }
        }

        public string DockState
        {
            get 
            { 
                MemoryStream stream = new MemoryStream();

                try
                {
                    SaveState( stream );
                }
                finally
                {
                    stream.Close();
                }

                return new String( Encoding.UTF8.GetChars( stream.GetBuffer()));
            }

            set
            {
                MemoryStream stream = new MemoryStream( Encoding.UTF8.GetBytes( value ));

                try
                {
                    LoadState( stream );
                }
                finally
                {
                    stream.Close();
                }
            }
        }

        public XmlElement DockStateXml
        {
            get 
            {
                MemoryStream stream = new MemoryStream();

                try
                {
                    SaveState( stream );
                    stream.Position = 0;
                    
                    XmlSerializer ser = new XmlSerializer( typeof( XmlElement ));
                    XmlElement xe = (XmlElement) ser.Deserialize( stream );

                    return xe;
                }
                finally
                {
                    stream.Close();
                }
            }

            set
            {
                MemoryStream stream = new MemoryStream( Encoding.UTF8.GetBytes( value.OuterXml ));
                stream.Position = 0;

                try
                {
                    LoadState( stream );
                }
                finally
                {
                    stream.Close();
                }
            }
        }


		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			try
			{
				if( disposing )
				{
					if(components != null)
					{
						components.Dispose();
					}
				}
				base.Dispose( disposing );
			}
			catch (Exception e)
			{
				Trace.WriteLine( "Exception in " + CLASS_NAME + ".Dispose()");
				Trace.WriteLine( e.ToString() );
				Trace.Flush();
			}
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            components = new Container();

            // 
            // DockHostControl
            // 
        }
        #endregion

        private void mDockingManager_DockStateUnavailable(object sender, DockStateUnavailableEventArgs arg)
        {

            // Dirty, Dirty HACK ALERT !!!!!!!
            // There is a bug in the docking manager. If we set the control visible here,
            // the docking manager makes it invisible right after return from this method.
            //
            // To work around this problem,  we'll BeginInvoke and do the SetDockVisibility from there,

            // Force the control to have a default dock state
            //mDockingManager.SetDockVisibility( arg.Control, true );
            //mDockingManager.DockControl( arg.Control, arg.Control, DockingStyle.Bottom, 250 );
            BeginInvoke( new DockStateUnavailableEventHandler( internal_DockStateUnavailableHandler ), new object[] { sender, arg } );
        }

        private void internal_DockStateUnavailableHandler(object sender, DockStateUnavailableEventArgs arg)
        {
            mDockingManager.SetDockVisibility( arg.Control, true );
        }

        private void mDockingManager_DockAllow(object sender, DockAllowEventArgs arg)
        {
            if( mFrozen )
            {
                arg.Cancel = true;
            }
        }

        private void mDockingManager_DragAllow(object sender, DragAllowEventArgs arg)
        {
            if( mFrozen )
                arg.Cancel = true;
        }

        private void control_TextChanged(object sender, EventArgs e)
        {
            Control control = (Control) sender;
            mDockingManager.SetDockLabel( control, control.Text );
        }
    }
}
